
/*
 * Lightning - A Weather Source API
 *
 * **This API is in BETA release.**  Global lightning archive, updated in real time. The Weather Source Lightning API provides the most accurate and reliable global lightning data available. All matching lightning events within 12 km (7 miles) of a provided location (and within the provided time range) are returned. The global lightning archive is available from the years 2021 and onward.  **IMPORTANT:** To authenticate to the Weather Source Lightning API, each request must contain your API key set to a custom header named `X-API-KEY`. If you do not have a Weather Source API key [sign up for a free 30-day developer account](https://developer.weathersourceapis.com/account/sign-up/). You may use the evaluation API Key `C0W60UOFRML47ytbXk4xlLBfv` to access example locations found in this documentation.  When using the \\\"Try it out\\\" capability in the documentation, the API key should be added by clicking the green \\\"Authorize\\\" button at the top of the page. 
 *
 * API version: 2.0.0
 * Contact: support@weathersource.com
 * Generated by: Swagger Codegen (https://github.com/swagger-api/swagger-codegen.git)
 */
package swagger

import (
	"context"
	"io/ioutil"
	"net/http"
	"net/url"
	"strings"
	"fmt"
	"time"
	"github.com/antihax/optional"
)

// Linger please
var (
	_ context.Context
)

type PointsApiService service
/*
PointsApiService Lightning data proximate to a Latitude/Longitude point.
Returns all lightning events that are both within 12 km (7 miles) of a provided Latitude/Longitude point and within the provided time range. The temporal range provided must not exceed 1 year in duration.  **IMPORTANT:** To authenticate to the Weather Source App Weather API, each request must contain your API key set to a custom header named &#x60;X-API-KEY&#x60;. If you do not have a Weather Source API key [sign up for a free 30-day developer account](https://developer.weathersourceapis.com/account/sign-up/). You may use the evaluation API Key &#x60;C0W60UOFRML47ytbXk4xlLBfv&#x60; to access example locations found in this documentation.  When using the \\\&quot;Try it out\\\&quot; capability in the documentation, the API key should be added by clicking the green \\\&quot;Authorize\\\&quot; button at the top of the page. 
 * @param ctx context.Context - for authentication, logging, cancellation, deadlines, tracing, etc. Passed from http.Request or context.Background().
 * @param latitude A latitude value between -90° and 90°.
 * @param longitude A longitude value between -180° (West) and 180° (East).
 * @param timestampStart A timestamp formatted as an RFC3339 date-time value that must occur temporally before timestampEnd. The UTC offset is required.
 * @param timestampEnd A timestamp formatted as an RFC3339 date-time value that must occur temporally after timestampStart. The UTC offset is required.
 * @param fields A comma separated list of field names to return. The timestamp field is always returned.  **Limiting the query to needed fields will improve the query response time.** 
 * @param optional nil or *PointsApiPointsLatitudelongitudePeriodTimestampStarttimestampEndGetOpts - Optional Parameters:
     * @param "UnitScale" (optional.String) -  The unit scale for returned values.&lt;br&gt;
     * @param "Limit" (optional.Int32) -  Limit the number of matching events returned. Returned events are ordered by distance from the provided location. Applying a limit will return the nearest events. A value of 0 means no limit. 
@return LightningPointObj
*/

type PointsApiPointsLatitudelongitudePeriodTimestampStarttimestampEndGetOpts struct {
    UnitScale optional.String
    Limit optional.Int32
}

func (a *PointsApiService) PointsLatitudelongitudePeriodTimestampStarttimestampEndGet(ctx context.Context, latitude float32, longitude float32, timestampStart time.Time, timestampEnd time.Time, fields []string, localVarOptionals *PointsApiPointsLatitudelongitudePeriodTimestampStarttimestampEndGetOpts) (LightningPointObj, *http.Response, error) {
	var (
		localVarHttpMethod = strings.ToUpper("Get")
		localVarPostBody   interface{}
		localVarFileName   string
		localVarFileBytes  []byte
		localVarReturnValue LightningPointObj
	)

	// create path and map variables
	localVarPath := a.client.cfg.BasePath + "/points/{latitude},{longitude}/period/{timestampStart},{timestampEnd}"
	localVarPath = strings.Replace(localVarPath, "{"+"latitude"+"}", fmt.Sprintf("%v", latitude), -1)
	localVarPath = strings.Replace(localVarPath, "{"+"longitude"+"}", fmt.Sprintf("%v", longitude), -1)
	localVarPath = strings.Replace(localVarPath, "{"+"timestampStart"+"}", fmt.Sprintf("%v", timestampStart), -1)
	localVarPath = strings.Replace(localVarPath, "{"+"timestampEnd"+"}", fmt.Sprintf("%v", timestampEnd), -1)

	localVarHeaderParams := make(map[string]string)
	localVarQueryParams := url.Values{}
	localVarFormParams := url.Values{}
	if latitude < -90.0 {
		return localVarReturnValue, nil, reportError("latitude must be greater than -90.0")
	}
	if latitude > 90.0 {
		return localVarReturnValue, nil, reportError("latitude must be less than 90.0")
	}
	if longitude < -180.0 {
		return localVarReturnValue, nil, reportError("longitude must be greater than -180.0")
	}
	if longitude > 180.0 {
		return localVarReturnValue, nil, reportError("longitude must be less than 180.0")
	}

	localVarQueryParams.Add("fields", parameterToString(fields, "csv"))
	if localVarOptionals != nil && localVarOptionals.UnitScale.IsSet() {
		localVarQueryParams.Add("unitScale", parameterToString(localVarOptionals.UnitScale.Value(), ""))
	}
	if localVarOptionals != nil && localVarOptionals.Limit.IsSet() {
		localVarQueryParams.Add("limit", parameterToString(localVarOptionals.Limit.Value(), ""))
	}
	// to determine the Content-Type header
	localVarHttpContentTypes := []string{}

	// set Content-Type header
	localVarHttpContentType := selectHeaderContentType(localVarHttpContentTypes)
	if localVarHttpContentType != "" {
		localVarHeaderParams["Content-Type"] = localVarHttpContentType
	}

	// to determine the Accept header
	localVarHttpHeaderAccepts := []string{"application/json"}

	// set Accept header
	localVarHttpHeaderAccept := selectHeaderAccept(localVarHttpHeaderAccepts)
	if localVarHttpHeaderAccept != "" {
		localVarHeaderParams["Accept"] = localVarHttpHeaderAccept
	}
	if ctx != nil {
		// API Key Authentication
		if auth, ok := ctx.Value(ContextAPIKey).(APIKey); ok {
			var key string
			if auth.Prefix != "" {
				key = auth.Prefix + " " + auth.Key
			} else {
				key = auth.Key
			}
			localVarHeaderParams["X-API-KEY"] = key
			
		}
	}
	r, err := a.client.prepareRequest(ctx, localVarPath, localVarHttpMethod, localVarPostBody, localVarHeaderParams, localVarQueryParams, localVarFormParams, localVarFileName, localVarFileBytes)
	if err != nil {
		return localVarReturnValue, nil, err
	}

	localVarHttpResponse, err := a.client.callAPI(r)
	if err != nil || localVarHttpResponse == nil {
		return localVarReturnValue, localVarHttpResponse, err
	}

	localVarBody, err := ioutil.ReadAll(localVarHttpResponse.Body)
	localVarHttpResponse.Body.Close()
	if err != nil {
		return localVarReturnValue, localVarHttpResponse, err
	}

	if localVarHttpResponse.StatusCode < 300 {
		// If we succeed, return the data, otherwise pass on to decode error.
		err = a.client.decode(&localVarReturnValue, localVarBody, localVarHttpResponse.Header.Get("Content-Type"));
		if err == nil { 
			return localVarReturnValue, localVarHttpResponse, err
		}
	}

	if localVarHttpResponse.StatusCode >= 300 {
		newErr := GenericSwaggerError{
			body: localVarBody,
			error: localVarHttpResponse.Status,
		}
		if localVarHttpResponse.StatusCode == 200 {
			var v LightningPointObj
			err = a.client.decode(&v, localVarBody, localVarHttpResponse.Header.Get("Content-Type"));
				if err != nil {
					newErr.error = err.Error()
					return localVarReturnValue, localVarHttpResponse, newErr
				}
				newErr.model = v
				return localVarReturnValue, localVarHttpResponse, newErr
		}
		if localVarHttpResponse.StatusCode == 0 {
			var v ErrorObj
			err = a.client.decode(&v, localVarBody, localVarHttpResponse.Header.Get("Content-Type"));
				if err != nil {
					newErr.error = err.Error()
					return localVarReturnValue, localVarHttpResponse, newErr
				}
				newErr.model = v
				return localVarReturnValue, localVarHttpResponse, newErr
		}
		return localVarReturnValue, localVarHttpResponse, newErr
	}

	return localVarReturnValue, localVarHttpResponse, nil
}
